@@ -99,25 +99,38 @@ HCPC <- function (res, nb.clust = 0, consol = TRUE, iter.max = 10, min = 3,
     cla <- NULL
 
     if (inherits(res, "PCA") | inherits(res, "MCA") | inherits(res,"MFA") | inherits(res, "HMFA") | inherits(res, "FAMD")) {
-	  if (kk<nrow(res$ind$coord)) res <- as.data.frame(res$ind$coord)
+	  if (kk<nrow(res$ind$coord)){
+	    res <- as.data.frame(res$ind$coord)
+		kk <- min(kk, nrow(unique(res$ind$coord)))
+      }
     }
     if (inherits(res, "CA")) {
 	  if (cluster.CA=="rows"){
-	    if (kk<nrow(res$row$coord)) res <- as.data.frame(sweep(res$row$coord,2,sqrt(res$eig[1:ncol(res$row$coord),1]),FUN="*"))
+	    if (kk<nrow(res$row$coord)) {
+		  res <- as.data.frame(sweep(res$row$coord,2,sqrt(res$eig[1:ncol(res$row$coord),1]),FUN="*"))
+		  kk <- min(kk, nrow(unique(res$row$coord)))
+        }
 	  } else {
-	    if (kk<nrow(res$col$coord)) res <- as.data.frame(sweep(res$col$coord,2,sqrt(res$eig[1:ncol(res$col$coord),1]),FUN="*"))
+	    if (kk<nrow(res$col$coord)){
+		  res <- as.data.frame(sweep(res$col$coord,2,sqrt(res$eig[1:ncol(res$col$coord),1]),FUN="*"))
+  		  kk <- min(kk, nrow(unique(res$col$coord)))
+        }
 	  }
     }
     if (is.data.frame(res)){
 	  res <-  res[,unlist(lapply(res,is.numeric)),drop=FALSE]
 ### AJOUT K-means
-	  if (kk<nrow(res)){
-	    cla <- kmeans(res,centers=kk,iter.max = 100, nstart = 4)
+      if (kk<nrow(res)) kk <- min(kk,nrow(unique(res)))
+	  if (kk <= nrow(res)){
+	    cla <- kmeans(res, centers=kk, iter.max = 100, nstart = 4)
         res <- PCA(cla$centers, row.w=cla$size, scale.unit = FALSE, ncp = Inf, graph = FALSE)
-      } else res <- PCA(res, scale.unit = FALSE, ncp = Inf, graph = FALSE)
+      } else {
+	    res <- PCA(res, scale.unit = FALSE, ncp = Inf, graph = FALSE)
+	  }
 ### Fin AJOUT K-means
 ##      res <- PCA(res, scale.unit = FALSE, ncp = Inf, graph = FALSE)
     }
+
     if(inherits(res,"CA")){
 	  aux <- res$eig
 	  if(cluster.CA=="rows") res <- PCA(res$row$coord, scale.unit = FALSE, ncp = Inf, graph = FALSE,row.w=res$call$marge.row*sum(res$call$X))
@@ -163,6 +176,8 @@ HCPC <- function (res, nb.clust = 0, consol = TRUE, iter.max = 10, min = 3,
             if (nb.clust == 0 | nb.clust == 1) nb.clust <- -1
         }
         if ((nb.clust == 0) | (nb.clust == 1)) {
+			  print("Click on the graph to cut the tree")
+			  flush.console()
 #            if (!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))){
               plot(t$tree, hang = -1, main = "Click to cut the tree",  xlab = "", sub = "")
               abline(h = auto.haut, col = "black", lwd = 3)
@@ -190,7 +205,6 @@ HCPC <- function (res, nb.clust = 0, consol = TRUE, iter.max = 10, min = 3,
 	X = as.data.frame(t$res$ind$coord)
 	ordColo = unique(clust[t$tree$order])
 
-
     if (graph) {
 #    if ((graph)&!nzchar(Sys.getenv("RSTUDIO_USER_IDENTITY"))) {
 #        rect <- rect.hclust(t$tree, h = y, border = seq(1, nb.clust, 1))
@@ -211,6 +225,7 @@ aux <- names(clust)
 ord <- order(res.consol$centers[,1,drop=FALSE])
 res.consol$centers <- res.consol$centers[ord,,drop=FALSE]
 clust <- (order(ord))[clust]
+
 ## Add 2014-07-08
 # if (kk<Inf){
   # rr <- as.factor(cla$cluster)
@@ -243,15 +258,16 @@ centers = res.consol$centers
 		## Add 2014-07-08
 		if (kk<Inf){
 		  clust <- clust[order(as.integer(names(clust)))]
-		  rr=clust[cla$cluster]
-		  names(rr)=names(cla$cluster)
-		  cla$cluster=as.factor(rr)
+		  rr <- clust[cla$cluster]
+		  names(rr) <- names(cla$cluster)
+		  cla$cluster <- as.factor(rr)
 		}
 ## fin ajout		
     }
     clust <- as.factor(clust)
 ## Add 2014-07-08
   X <- cbind.data.frame(X,clust)
+
 if (kk<Inf){
   if (inherits(res.sauv, "PCA") | inherits(res.sauv, "MCA") | inherits(res.sauv,"MFA") | inherits(res.sauv, "HMFA") | inherits(res.sauv, "FAMD")){
     if (is.null(res.sauv$call$ind.sup)) data.clust <- cbind.data.frame(res.sauv$call$X, clust=cla$cluster)
@@ -307,22 +323,22 @@ if (description){
   if (inherits(res.sauv, "CA")&(cluster.CA=="columns")) tabInd <- cbind.data.frame(res.sauv$col$coord,data.clust[rownames(res.sauv$col$coord),ncol(data.clust)])
 colnames(tabInd)[ncol(tabInd)]="Cluster"
 
-list.centers <- by(tabInd[,-ncol(tabInd),drop=FALSE], tabInd[,ncol(tabInd)], colMeans)
-centers <- matrix(unlist(list.centers), ncol = ncol(tabInd)-1,byrow = TRUE)
-colnames(centers) = colnames(tabInd)[-ncol(tabInd)]
-cluster <- tabInd[,ncol(tabInd),drop=FALSE]
-para <- by(tabInd, cluster, simplify = FALSE, select, default.size = nb.par, method = metric, coord.centers = centers)
-dist <- by(tabInd, cluster, simplify = FALSE, distinctivness, default.size = nb.par, method = metric, coord.centers = centers)
-desc.ind <- list(para = para, dist = dist)
+if (description){
+  list.centers <- by(tabInd[,-ncol(tabInd),drop=FALSE], tabInd[,ncol(tabInd)], colMeans)
+  centers <- matrix(unlist(list.centers), ncol = ncol(tabInd)-1,byrow = TRUE)
+  colnames(centers) = colnames(tabInd)[-ncol(tabInd)]
+  cluster <- tabInd[,ncol(tabInd),drop=FALSE]
+  para <- by(tabInd, cluster, simplify = FALSE, select, default.size = nb.par, method = metric, coord.centers = centers)
+  dist <- by(tabInd, cluster, simplify = FALSE, distinctivness, default.size = nb.par, method = metric, coord.centers = centers)
+  desc.ind <- list(para = para, dist = dist)
+}
     if (consol) call <- list(t = t, min = min, max = max, X = X, bw.before.consol=sum(rev(t$tree$height)[1:(nb.clust-1)]),bw.after.consol=res.consol$betweenss/nrow(data.clust),vec = vec,call=match.call())
 	else call <- list(t = t, min = min, max = max, X = X, bw.before.consol=sum(rev(t$tree$height)[1:(nb.clust-1)]),vec = vec,call=match.call())
-#    call <- list(t = t, min = min, max = max, X = data.clust, vec = vec,call=sys.calls()[[1]])
 	if (description){
-	  if (kk!=Inf) res.HCPC <- list(data.clust = data.clust, desc.var = desc.var, call = call, desc.ind = desc.ind)
-      else res.HCPC <- list(data.clust = data.clust, desc.var = desc.var, desc.axes = desc.axe, call = call, desc.ind = desc.ind)
+	  if (kk!=Inf) res.HCPC <- list(data.clust = data.clust, desc.var = desc.var, desc.ind = desc.ind, call = call)
+      else res.HCPC <- list(data.clust = data.clust, desc.var = desc.var, desc.axes = desc.axe, desc.ind = desc.ind, call = call)
 	} else {
-	  if (kk!=Inf) res.HCPC <- list(data.clust = data.clust, call = call, desc.ind = desc.ind)
-      else res.HCPC <- list(data.clust = data.clust, call = call, desc.ind = desc.ind)
+	  res.HCPC <- list(data.clust = data.clust, call = call)
 	}
     if ((kk==Inf)&(graph)) {
 #       plot.HCPC(res.HCPC,choice="tree",new.plot=FALSE)
