@@ -16,18 +16,17 @@ MFA <- function (base, group, type = rep("s",length(group)), excl = NULL, ind.su
       s <- sum(wt)
 	  wt <- wt/s
       center <- colSums(wt * x)
-      x <- sqrt(wt) * sweep(t(t(x)*sqrt(ponderation.x)), 2, center, check.margin = FALSE)
+      x <- sqrt(wt) * t(t(sweep(x, 2, center, check.margin = FALSE))*sqrt(ponderation.x))
       center <- colSums(wt * y)
-      y <- sqrt(wt) * sweep(t(t(y)*sqrt(ponderation.y)), 2, center, check.margin = FALSE)
+      y <- sqrt(wt) * t(t(sweep(y, 2, center, check.margin = FALSE))*sqrt(ponderation.y))
 	  Lg <- 0
 	  for (i in 1:ncol(x)) Lg <- Lg+sum(crossprod(x[,i],y)^2)
 	  Lg
     }
-
 if (!is.null(tab.comp)){
   if (!is.null(weight.col.mfa)) stop("Weightings on the variables are not allowed with the tab.comp argument")
-  if (!is.null(ind.sup)) stop("Supplementary individuals are not allowed with tab.comp")
-  if (!is.null(num.group.sup)) stop("Supplementary groups are not allowed with tab.comp")
+#  if (!is.null(ind.sup)) stop("Supplementary individuals are not allowed with tab.comp")
+#  if (!is.null(num.group.sup)) stop("Supplementary groups are not allowed with tab.comp")
 }
 
 nature.group <- NULL
@@ -57,6 +56,8 @@ for (i in 1:length(group)){
 #    for (j in 1:ncol(base)) if (colnames(base)[j]=="") colnames(base)[j] = paste("V",j,sep="")
 #    for (j in 1:nrow(base)) if (is.null(rownames(base)[j])) rownames(base)[j] = paste("row",j,sep="")
     base <- as.data.frame(base)
+    is.quali <- which(!unlist(lapply(base,is.numeric)))
+    base[,is.quali] <- lapply(base[,is.quali,drop=FALSE],as.factor)
 	base <- droplevels(base)
     if (!is.null(ind.sup)) {
       base <- rbind.data.frame(base[-ind.sup,],base[ind.sup,,drop=FALSE])
@@ -85,7 +86,7 @@ for (i in 1:length(group)){
     base <- as.data.frame(base)
     if (!inherits(base, "data.frame")) stop("base should be a data.frame")
     res.separe <- vector(mode = "list", length = nbre.group)
-    if (is.null(name.group)) name.group <- paste("group", 1:nbre.group, sep = ".")
+    if (is.null(name.group)) name.group <- paste0("Gr", 1:nbre.group)
     names(res.separe) <- name.group
     ind.grpe <- 0
     if (any(is.na(base))){
@@ -300,27 +301,39 @@ if (!is.null(tab.comp)){
         if (type[g] == "n")  ind.quali <- c(ind.quali, c((ind.var + 1):(ind.var + group[g])))
         ind.var <- ind.var + group[g]
     }
-    aux.quali.sup.indice <- aux.quali.sup <- data.sup <- NULL
+    aux_quali_sup_indice <- aux_quali_sup <- data.sup <- NULL
     if (!is.null(ind.quali)){
-      aux.quali.sup <- as.data.frame(base[, ind.quali,drop=FALSE])
-      if (is.null(data.group.sup)) aux.quali.sup.indice <- (ncol(data)+1):(ncol(data)+ncol(aux.quali.sup))
-      else aux.quali.sup.indice <- (ncol(data)+ncol(data.group.sup)+1):(ncol(data)+ncol(data.group.sup)+ncol(aux.quali.sup))
-      data.pca <- cbind.data.frame(data.pca,aux.quali.sup)
+      aux_quali_sup <- as.data.frame(base[, ind.quali,drop=FALSE])
+      if (is.null(data.group.sup)) aux_quali_sup_indice <- (ncol(data)+1):(ncol(data)+ncol(aux_quali_sup))
+      else aux_quali_sup_indice <- (ncol(data)+ncol(data.group.sup)+1):(ncol(data)+ncol(data.group.sup)+ncol(aux_quali_sup))
+      data.pca <- cbind.data.frame(data.pca,aux_quali_sup)
     }
 row.w = row.w[1:nb.actif]
 ###  Begin handle missing values
 if ((!is.null(tab.comp))&(any("n"%in%type))){
-  data.pca <- data.pca[,-aux.quali.sup.indice]
-  aux.quali.sup.indice <- NULL
+  data.pca <- data.pca[,-aux_quali_sup_indice]
+  aux_quali_sup_indice <- NULL
 }
 ###  End handle missing values
- res.globale <- PCA(data.pca, scale.unit = FALSE, col.w = ponderation, row.w=row.w,ncp = ncp, ind.sup = ind.sup, quali.sup = aux.quali.sup.indice, quanti.sup = data.group.sup.indice, graph = FALSE)
-# res.globale <- PCA(data.pca, scale.unit = FALSE, col.w = ponderation, row.w=row.w,ncp = ncp.tmp, ind.sup = ind.sup, quali.sup = aux.quali.sup.indice, quanti.sup = data.group.sup.indice, graph = FALSE)
+ res.globale <- PCA(data.pca, scale.unit = FALSE, col.w = ponderation, row.w=row.w,ncp = ncp, ind.sup = ind.sup, quali.sup = aux_quali_sup_indice, quanti.sup = data.group.sup.indice, graph = FALSE)
+# res.globale <- PCA(data.pca, scale.unit = FALSE, col.w = ponderation, row.w=row.w,ncp = ncp.tmp, ind.sup = ind.sup, quali.sup = aux_quali_sup_indice, quanti.sup = data.group.sup.indice, graph = FALSE)
 ###  Begin handle missing values
+
 if ((!is.null(tab.comp))&(any("n"%in%type))){
-  res.globale$quali.var$coord <- res.globale$var$coord[unlist(ind.var.group[type%in%"n"]),]
-  res.globale$quali.var$contrib <- res.globale$var$contrib[unlist(ind.var.group[type%in%"n"]),]
-  res.globale$quali.var$cos2 <- res.globale$var$cos2[unlist(ind.var.group[type%in%"n"]),]
+  # res.globale$quali.var$coord <- res.globale$var$coord[unlist(ind.var.group[type%in%"n"]),]
+  # res.globale$quali.var$contrib <- res.globale$var$contrib[unlist(ind.var.group[type%in%"n"]),]
+  # res.globale$quali.var$cos2 <- res.globale$var$cos2[unlist(ind.var.group[type%in%"n"]),]
+  if (!is.null(type.var%in%"quali")) {
+    if (any(type.var%in%c("freq","quanti"))){
+	  res.globale$quali.var$coord <- res.globale$var$coord[-(1:sum(type.var%in%c("freq","quanti"))),]
+      res.globale$quali.var$cos2 <- res.globale$var$cos2[-(1:sum(type.var%in%c("freq","quanti"))),]
+      res.globale$quali.var$contrib <- res.globale$var$contrib[-(1:sum(type.var%in%c("freq","quanti"))),]
+	} else{
+	  res.globale$quali.var$coord <- res.globale$var$coord
+      res.globale$quali.var$cos2 <- res.globale$var$cos2
+      res.globale$quali.var$contrib <- res.globale$var$contrib
+	}
+  }
   res.globale$call$quali.sup$barycentre <- sweep(crossprod(tab.comp[,unlist(ind.var.group[type%in%"n"])],as.matrix(data.pca)),1,apply(tab.comp[,unlist(ind.var.group[type%in%"n"])],2,sum),FUN="/")
   res.globale$quali.sup$coord <- sweep(crossprod(tab.comp[,unlist(ind.var.group[type%in%"n"])],res.globale$ind$coord),1,apply(tab.comp[,unlist(ind.var.group[type%in%"n"])],2,sum),FUN="/")
 }
@@ -350,14 +363,10 @@ if ((!is.null(tab.comp))&(any("n"%in%type))){
     }
     coord.group <- t(t(contrib.group)*res.globale$eig[1:ncol(contrib.group),1])
     cos2.group <- coord.group^2/dist2.group
-
+	
     if (!is.null(num.group.sup)){
       coord.group.sup <- matrix(NA, length(num.group.sup), ncp)
       dimnames(coord.group.sup) <- list(name.group[num.group.sup], paste("Dim", c(1:ncp), sep = "."))
-      # if (is.null(ind.sup)) tab <- (cov.wt(cbind.data.frame(res.globale$ind$coord,data.group.sup),wt=row.w/sum(row.w),method="ML")$cov)^2
-        # else tab <- (cov.wt(cbind.data.frame(res.globale$ind$coord,data.group.sup[-ind.sup, ]),wt=row.w/sum(row.w),method="ML")$cov)^2
-      # tab <- t(t(tab)*c(1/res.globale$eig[1:ncol(res.globale$ind$coord),1],ponderation.group.sup))
-      # tab <- tab*c(1/res.globale$eig[1:ncol(res.globale$ind$coord),1],ponderation.group.sup)
       ind.gc <- 0
       for (gc in 1:length(num.group.sup)) {
         # if (group.mod[num.group.sup[gc]]!=1) coord.group.sup[gc,] <- apply(tab[1:ncp.tmp,  (ncp.tmp+ind.gc+1):(ncp.tmp+ind.gc+group.mod[num.group.sup[gc]])],1,sum)
@@ -369,16 +378,6 @@ if ((!is.null(tab.comp))&(any("n"%in%type))){
         ind.gc <- ind.gc + group.mod[num.group.sup[gc]]
       }
     }
-   # if (is.null(num.group.sup)){
-     # if (is.null(ind.sup)) tab <- (cov.wt(t(t(data)*sqrt(ponderation)),wt=row.w/sum(row.w),method="ML")$cov)^2
-       # else tab <- (cov.wt(t(t(data[-ind.sup, ])*sqrt(ponderation)),wt=row.w/sum(row.w),method="ML")$cov)^2
-   # }
-   # else{
-     # if (is.null(ind.sup)) tab <- (cov.wt(cbind.data.frame(data, data.group.sup),wt=row.w/sum(row.w),method="ML")$cov)^2
-       # else tab <- (cov.wt(cbind.data.frame(data[-ind.sup, ], data.group.sup[-ind.sup,]),wt=row.w/sum(row.w),method="ML")$cov)^2
-     # tab <- t(t(tab)*c(ponderation,ponderation.group.sup))
-     # tab <- tab*c(ponderation,ponderation.group.sup)
-   # }
     Lg <- matrix(0, nbre.group+1, nbre.group+1)
     ind.gl <- 0
     for (gl in c(group.actif,num.group.sup)) {
@@ -420,7 +419,6 @@ if ((!is.null(tab.comp))&(any("n"%in%type))){
     }
     res.ind.partiel <- vector(mode = "list", length = nbre.group)
     names(res.ind.partiel) <- name.group
-
     for (g in group.actif){
       Xis <- t(t(as.matrix(data.partiel[[g]]))-res.globale$call$centre) 
       Xis <- t(t(Xis)/res.globale$call$ecart.type)
@@ -448,9 +446,20 @@ if ((!is.null(tab.comp))&(any("n"%in%type))){
       res.groupes$cos2.sup <- coord.group.sup[,1:ncp,drop=FALSE]^2/dist2.group.sup
       res.groupes$dist2.sup <- dist2.group.sup
     }
-    inertie.intra.ind.partiel <- matrix(NA, (nb.actif * length(group.actif) ), ncp)
-    nom.ligne <- NULL
-    for (i in 1:nb.actif) nom.ligne <- c(nom.ligne, paste(rownames(base)[i], name.group[group.actif], sep = "."))
+  ####CHANGED THIS!!!! ------------------
+  # OLD#
+  # nom.ligne <- NULL
+  # for (i in 1:nb.actif) nom.ligne <- c(nom.ligne, paste(rownames(base)[i], name.group[group.actif], sep = "."))
+  # NEW#
+  nom.ligne <- 
+    c(
+      sapply(
+        rownames(base)[1:nb.actif], 
+        paste, 
+        name.group[group.actif], 
+        sep = ".")
+    )
+
     tmp <- array(0,dim=c(nrow(res.globale$ind$coord),ncp,length(group.actif)))
     for (g in 1:length(group.actif)) tmp[,,g] <- (res.ind.partiel[[group.actif[g]]]$coord.sup[1:nb.actif,1:ncp,drop=FALSE]-res.globale$ind$coord[,1:ncp,drop=FALSE])^2/length(group.actif)
 ## Ajour Avril 2011
@@ -458,7 +467,23 @@ tmp <- tmp*row.w
     variab.auxil <- apply(tmp,2,sum)   ## attention, array
     tmp <- sweep(tmp,2,variab.auxil,FUN="/") * 100  ## attention, array
     inertie.intra.ind <- apply(tmp,c(1,2),sum)
-    for (i in 1:nb.actif) inertie.intra.ind.partiel[((i - 1) * length(group.actif)  + 1):(i * length(group.actif)), ] <- t(tmp[i,1:ncp,])
+  ####CHANGED THIS!!!! --------------------
+  #OLD#
+  #  inertie.intra.ind.partiel <- as.data.frame(matrix(NA, (nb.actif * length(group.actif)), ncp.tmp))
+#  for (i in 1:nb.actif) inertie.intra.ind.partiel[((i - 1) * length(group.actif)  + 1):(i * length(group.actif)), ] <- t(tmp[i,1:ncp,])
+  #NEW#
+  inertie.intra.ind.partiel <- 
+    data.frame(
+      do.call(
+        rbind, 
+        lapply(1:dim(tmp)[1], function(x) t(tmp[x, 1:ncp, ]))
+      )
+    )
+  inertie.intra.ind.partiel <- as.matrix(inertie.intra.ind.partiel)
+  # xxx <- as.data.frame(matrix(NA, (nb.actif * length(group.actif)), ncp))
+  # colnames(inertie.intra.ind.partiel) <- colnames(xxx)
+
+
     rownames(inertie.intra.ind) <- rownames(res.globale$ind$coord)
     rownames(inertie.intra.ind.partiel) <- nom.ligne
     colnames(inertie.intra.ind) <- colnames(inertie.intra.ind.partiel) <- paste("Dim", c(1:ncp), sep = ".")
@@ -549,11 +574,21 @@ tmp <- tmp*row.w
         }
     }
    eig <- res.globale$eig
-    nom.ligne <- NULL
-    for (i in 1:nbre.ind) {
-        ind.tmp <- rownames(base)[i]
-        nom.ligne <- c(nom.ligne, paste(ind.tmp, name.group[group.actif], sep = "."))
-    }
+  ## CHANGED THIS!!!! ---------------
+  # nom.ligne <- NULL
+  # for (i in 1:nbre.ind) {
+  #   ind.tmp <- rownames(base)[i]
+  #   nom.ligne <- c(nom.ligne, paste(ind.tmp, name.group[group.actif], sep = "."))
+  # }
+  
+  nom.ligne <- 
+    c(
+      sapply(
+        rownames(base), 
+        paste, 
+        name.group[group.actif], 
+        sep = ".")
+    )
     coord.ind.partiel <- matrix(NA, (nbre.ind * length(group.actif)), ncp)
     rownames(coord.ind.partiel) <- nom.ligne
     colnames(coord.ind.partiel) <- paste("Dim", c(1:ncp), sep = ".")
@@ -594,12 +629,12 @@ tmp <- tmp*row.w
       inertie.intra.cg.partiel <- matrix(NA, (nrow(barycentre) * length(group.actif) ), ncp)
       tmp <- array(0,dim=c(nrow(res.globale$quali.sup$coord),ncp,length(group.actif)))
       ind.col <- 0
-      for (g in 1:length(group.actif)) {
-        cg.partiel <- as.data.frame(matrix(res.globale$call$centre, nrow(barycentre), ncol(barycentre), byrow = TRUE, dimnames = dimnames(barycentre)))
+ for (g in 1:length(group.actif)) {
+        # cg.partiel <- as.data.frame(matrix(res.globale$call$centre, nrow(barycentre), ncol(barycentre), byrow = TRUE, dimnames = dimnames(barycentre)))
+        cg.partiel <- as.data.frame(matrix(res.globale$call$centre[sum(group.mod[group.actif[1:length(group.actif)]])], nrow(barycentre), sum(group.mod[group.actif[1:length(group.actif)]]), byrow = TRUE, dimnames = dimnames(barycentre[,1:sum(group.mod[group.actif[1:length(group.actif)]]),drop=FALSE])))
         cg.partiel[, (ind.col + 1):(ind.col + group.mod[group.actif[g]])] <- barycentre[, (ind.col + 1):(ind.col + group.mod[group.actif[g]])]
         ind.col <- ind.col + group.mod[group.actif[g]]
         Xis <- t((t(cg.partiel)-res.globale$call$centre)/res.globale$call$ecart.type)
-##        coord.quali.sup <- length(group.actif) * as.matrix(Xis)%*%diag((res.globale$call$col.w))%*%res.globale$svd$V
         coord.quali.sup <- length(group.actif) * as.matrix(Xis)
         coord.quali.sup <- t(t(coord.quali.sup)*res.globale$call$col.w)
         coord.quali.sup <- crossprod(t(coord.quali.sup),res.globale$svd$V)
@@ -761,7 +796,6 @@ cor.partial.axes <- cov.wt(aux,wt=row.w/sum(row.w),method="ML",cor=TRUE)$cor
     resultats$partial.axes = res.partial.axes
     resultats$call = call
 	resultats$call$call <- match.call()
-#	resultats$call$call <- sys.calls()[[1]]
     resultats$global.pca = res.globale
     class(resultats) <- c("MFA", "list")
 
@@ -776,15 +810,15 @@ cor.partial.axes <- cov.wt(aux,wt=row.w/sum(row.w),method="ML",cor=TRUE)$cor
           max.inertia <- order(apply(resultats$quali.var.sup$within.inertia[,1:2],1,sum))
           cg.plot.partial <- c(cg.plot.partial,rownames(resultats$quali.var.sup$coord)[max.inertia[1:length(max.inertia)]])
         }
-        plot.MFA(resultats,choix="ind",invisible="ind",partial=cg.plot.partial,habillage="group",axes=axes,new.plot=TRUE)
+        print(plot.MFA(resultats,choix="ind",invisible="ind",partial=cg.plot.partial,habillage="group",axes=axes,new.plot=TRUE))
       }
       max.inertia <- order(apply(resultats$ind$within.inertia[,1:2],1,sum))
-      plot.MFA(resultats,choix="axes",habillage="group",axes=axes,new.plot=TRUE,shadowtext=TRUE)
-      plot.MFA(resultats,choix="ind",invisible="quali",partial=rownames(resultats$ind$coord)[max.inertia[c(1:2,nrow(resultats$ind$coord)-1,nrow(resultats$ind$coord))]],habillage="group",axes=axes,new.plot=TRUE)
-      if ("c"%in%type) plot.MFA(resultats,choix="var",habillage="group",axes=axes,new.plot=TRUE,shadowtext=TRUE	)
-      if ("f"%in%type) plot.MFA(resultats,choix="freq",habillage="group",axes=axes,new.plot=TRUE)
-      plot.MFA(resultats,choix="ind",invisible="quali",habillage = "ind",axes=axes,new.plot=TRUE,col.hab=1+3*(1:nbre.ind)%in%ind.sup)
-      plot.MFA(resultats,choix="group",axes=axes,new.plot=TRUE)
+      print(plot.MFA(resultats,choix="axes",habillage="group",axes=axes,new.plot=TRUE,shadowtext=TRUE))
+      print(plot.MFA(resultats,choix="ind",invisible="quali",partial=rownames(resultats$ind$coord)[max.inertia[c(1:2,nrow(resultats$ind$coord)-1,nrow(resultats$ind$coord))]],habillage="group",axes=axes,new.plot=TRUE))
+      if ("c"%in%type) print(plot.MFA(resultats,choix="var",habillage="group",axes=axes,new.plot=TRUE,shadowtext=TRUE))
+      if ("f"%in%type) print(plot.MFA(resultats,choix="freq",habillage="group",axes=axes,new.plot=TRUE))
+      print(plot.MFA(resultats,choix="ind",invisible="quali",habillage = "ind",axes=axes,new.plot=TRUE,col.hab=1+3*(1:nbre.ind)%in%ind.sup))
+      print(plot.MFA(resultats,choix="group",axes=axes,new.plot=TRUE))
     }
     return(resultats)
 }
